
package com.ibm.developerworks.java.j2me_tutorial;

import com.sun.kjava.Bitmap;
import com.sun.kjava.Button;
import com.sun.kjava.Dialog;
import com.sun.kjava.Graphics;
import com.sun.kjava.HelpDisplay;
import com.sun.kjava.Spotlet;


/**
 * Scribble is a self-contained, demonstration application. It demonstrates the
 * use of Sun's KVM for the PalmOS.
 *
 * <p>Scribble is a small application that allows you to draw freehand pictures
 * on the screen, adding regular text if necessary.</p>
 *
 * <p>Copyright IBM developerWorks, 2001</p>
 * @author Steven Gould
 */
public class Scribble extends Spotlet
   {
   /** The coordinate of the left hand edge of the drawing frame. */
   final static int FRAME_LEFT   = 0;

   /** The coordinate of the top edge of the drawing frame. */
   final static int FRAME_TOP    = 0;

   /** The width of the drawing frame. */
   final static int FRAME_WIDTH  = 159;

   /** The height of the drawing frame. */
   final static int FRAME_HEIGHT = 140;

   /** Handle to the singleton Graphics object */
   static Graphics g = Graphics.getGraphics();

   /** The X coordinate of the last point the user touched the screen. */
   int lastX;

   /** The Y coordinate of the last point the user touched the screen. */
   int lastY;

   /** A reference to the Exit button on the screen. */
   Button exitButton;

   /** A reference to the Clear button on the screen. */
   Button clearButton;

   /** The last character entered. */
   char lastKey;

   /**
    * The current string being written. This is needed to support
    * backspace; i.e. to allow the user to delete characters just typed.
    */
   StringBuffer currentText;

   /**
    * The main method simply creates a Scribble spotlet and
    * registers its event handlers.
    */
   public static void main(String[] args)
   {
      String helpText = "Scribble lets you draw or scribble pictures and/or diagrams on the screen with your stylus.";

      HelpDisplay helpDisplay
         = new HelpDisplay(helpText, "com.ibm.developerworks.java.j2me_tutorial.Scribble",
             NO_EVENT_OPTIONS);
      helpDisplay.register(NO_EVENT_OPTIONS);
   }

   /**
    * Default constructor creates the GUI components and draws them.
    */
   public Scribble()
   {
      lastX = 0;
      lastY = 0;
      currentText = new StringBuffer();

      // create the buttons
      clearButton = new Button( "Clear",105, 145 );
      exitButton  = new Button( "Exit", 135, 145 );

      g.clearScreen();

      paint();
   }

   /**
    * Displays the drawing frame and the controls.
    */
   private void paint()
   {
      g.resetDrawRegion();

      // Draw the frame and clear the space below it
      g.drawBorder( FRAME_LEFT+1, FRAME_TOP+1,
               FRAME_LEFT+FRAME_WIDTH-2, FRAME_TOP+FRAME_HEIGHT-2,
               g.PLAIN, g.SIMPLE );

      // Draw any text entered
      switch ( lastKey )
      {
         case 8:  // Backspace
            if ( currentText.length() == 0 )
               break;

            char charToDelete
               = currentText.charAt( currentText.length()-1 );
            currentText.deleteCharAt( currentText.length()-1 );

            // Calculate region to erase
            int width = g.getWidth(""+charToDelete);
            int height = g.getHeight(""+charToDelete);
            lastX -= width;
            g.drawRectangle( lastX, lastY-height, width, height,
                   g.ERASE, 38 );
            break;

         default:
            g.drawString(""+lastKey,lastX,lastY);
            lastX += g.getWidth(""+lastKey);
            currentText.append(lastKey);
      }

      // Draw GUI controls and buttons
      clearButton.paint();
      exitButton.paint();

      // constrain the drawing region to within the frame
      g.setDrawRegion(FRAME_LEFT+1,FRAME_TOP+1,FRAME_WIDTH-2,FRAME_HEIGHT-2);
   }

   /**
    * Handle a pen down event.
    */
   public void penDown(int x, int y)
   {
      if (clearButton.pressed(x,y))
      {
         clearDrawingArea();
         paint();
         return;
      }

      if (exitButton.pressed(x,y))
      {
         System.exit(0);
      }

      lastX = x;
      lastY = y;
      currentText.delete(0, currentText.length());
   }

   /**
    * Handle a key down event.
    */
   public void keyDown(int keyCode)
   {
      lastKey = (char)keyCode;
      paint();
   }

   /**
    * Handle a pen move event.
    */
   public void penMove(int x, int y)
   {
      if (FRAME_LEFT < x && FRAME_LEFT+FRAME_WIDTH > x &&
         FRAME_TOP < y && FRAME_TOP+FRAME_HEIGHT > y)
      {
         g.drawLine(lastX,lastY,x,y,g.PLAIN);
         lastX = x;
         lastY = y;
      }
   }

   /**
    * Clears the drawing area by sliding the contents off to the
    * side (left or right). The side to which the contents "disappear"
    * depends on where the last point drawn was. Basically, if the last
    * point drawn was on the left (or right) hand side of the drawing
    * area, then the contents are moved off to the right (or left,
    * respectively).
    */
   private void clearDrawingArea()
   {
      int dx = (lastX > FRAME_WIDTH/2) ? -1 : +1;

      int startX;
      int endX;

      if ( dx > 0 )
         startX = FRAME_LEFT+1;
      else
         startX = FRAME_LEFT+FRAME_WIDTH-1;

      // Clear left or right hand edge (one pixel wide)
      g.drawLine( startX, FRAME_TOP+1,
                   startX, FRAME_TOP+FRAME_HEIGHT-2, g.ERASE);

      // Slide the drawing area contents off to the side
      for ( int x = 0; x < FRAME_WIDTH; x++ )
         g.copyRegion( 1, FRAME_TOP+1,
                  FRAME_WIDTH-2, FRAME_HEIGHT-2,
                  1+dx, FRAME_TOP+1, g.OVERWRITE );
   }
}
